@@ -16,7 +16,7 @@
  * Inc.,  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <libxfs.h>
+#include "libxfs.h"
 #include "xfs_metadump.h"
 
 char 		*progname;
@@ -60,8 +60,9 @@ perform_restore(
 	__be64			*block_index;
 	char			*block_buffer;
 	int			block_size;
-	int			max_indicies;
+	int			max_indices;
 	int			cur_index;
+	int			mb_count;
 	xfs_metablock_t		tmb;
 	xfs_sb_t		sb;
 	__int64_t		bytes_read;
@@ -79,17 +80,15 @@ perform_restore(
 		fatal("specified file is not a metadata dump\n");
 
 	block_size = 1 << tmb.mb_blocklog;
-	max_indicies = (block_size - sizeof(xfs_metablock_t)) / sizeof(__be64);
+	max_indices = (block_size - sizeof(xfs_metablock_t)) / sizeof(__be64);
 
-	metablock = (xfs_metablock_t *)calloc(max_indicies + 1, block_size);
+	metablock = (xfs_metablock_t *)calloc(max_indices + 1, block_size);
 	if (metablock == NULL)
 		fatal("memory allocation failure\n");
 
-	metablock->mb_count = be16_to_cpu(tmb.mb_count);
-	metablock->mb_blocklog = tmb.mb_blocklog;
-
-	if (metablock->mb_count == 0 || metablock->mb_count > max_indicies)
-		fatal("bad block count: %u\n", metablock->mb_count);
+	mb_count = be16_to_cpu(tmb.mb_count);
+	if (mb_count == 0 || mb_count > max_indices)
+		fatal("bad block count: %u\n", mb_count);
 
 	block_index = (__be64 *)((char *)metablock + sizeof(xfs_metablock_t));
 	block_buffer = (char *)metablock + block_size;
@@ -101,16 +100,26 @@ perform_restore(
 		fatal("first block is not the primary superblock\n");
 
 
-	if (fread(block_buffer, metablock->mb_count << metablock->mb_blocklog,
+	if (fread(block_buffer, mb_count << tmb.mb_blocklog,
 			1, src_f) != 1)
 		fatal("error reading from file: %s\n", strerror(errno));
 
-	libxfs_xlate_sb(block_buffer, &sb, 1, XFS_SB_ALL_BITS);
+	libxfs_sb_from_disk(&sb, (xfs_dsb_t *)block_buffer);
 
 	if (sb.sb_magicnum != XFS_SB_MAGIC)
 		fatal("bad magic number for primary superblock\n");
 
-	((xfs_sb_t*)block_buffer)->sb_inprogress = 1;
+	/*
+	 * Normally the upper bound would be simply XFS_MAX_SECTORSIZE
+	 * but the metadump format has a maximum number of BBSIZE blocks
+	 * it can store in a single metablock.
+	 */
+	if (sb.sb_sectsize < XFS_MIN_SECTORSIZE ||
+	    sb.sb_sectsize > XFS_MAX_SECTORSIZE ||
+	    sb.sb_sectsize > max_indices * block_size)
+		fatal("bad sector size %u in metadump image\n", sb.sb_sectsize);
+
+	((xfs_dsb_t*)block_buffer)->sb_inprogress = 1;
 
 	if (is_target_file)  {
 		/* ensure regular files are correctly sized */
@@ -121,7 +130,7 @@ perform_restore(
 	} else  {
 		/* ensure device is sufficiently large enough */
 
-		char		*lb[XFS_MAX_SECTORSIZE] = { 0 };
+		char		*lb[XFS_MAX_SECTORSIZE] = { NULL };
 		off64_t		off;
 
 		off = sb.sb_dblocks * sb.sb_blocksize - sizeof(lb);
@@ -134,36 +143,34 @@ perform_restore(
 
 	for (;;) {
 		if (show_progress && (bytes_read & ((1 << 20) - 1)) == 0)
-			print_progress("%lld MB read\n", bytes_read >> 20);
+			print_progress("%lld MB read", bytes_read >> 20);
 
-		for (cur_index = 0; cur_index < metablock->mb_count; cur_index++) {
+		for (cur_index = 0; cur_index < mb_count; cur_index++) {
 			if (pwrite64(dst_fd, &block_buffer[cur_index <<
-						metablock->mb_blocklog],
-					block_size,
+					tmb.mb_blocklog], block_size,
 					be64_to_cpu(block_index[cur_index]) <<
 						BBSHIFT) < 0)
 				fatal("error writing block %llu: %s\n",
 					be64_to_cpu(block_index[cur_index]) << BBSHIFT,
 					strerror(errno));
 		}
-		if (metablock->mb_count < max_indicies)
+		if (mb_count < max_indices)
 			break;
 
 		if (fread(metablock, block_size, 1, src_f) != 1)
 			fatal("error reading from file: %s\n", strerror(errno));
 
-		if (metablock->mb_count == 0)
+		mb_count = be16_to_cpu(metablock->mb_count);
+		if (mb_count == 0)
 			break;
+		if (mb_count > max_indices)
+			fatal("bad block count: %u\n", mb_count);
 
-		metablock->mb_count = be16_to_cpu(metablock->mb_count);
-		if (metablock->mb_count > max_indicies)
-			fatal("bad block count: %u\n", metablock->mb_count);
-
-		if (fread(block_buffer, metablock->mb_count <<
-				 metablock->mb_blocklog, 1, src_f) != 1)
+		if (fread(block_buffer, mb_count << tmb.mb_blocklog,
+								1, src_f) != 1)
 			fatal("error reading from file: %s\n", strerror(errno));
 
-		bytes_read += block_size;
+		bytes_read += block_size + (mb_count << tmb.mb_blocklog);
 	}
 
 	if (progress_since_warning)
@@ -171,7 +178,12 @@ perform_restore(
 
 	memset(block_buffer, 0, sb.sb_sectsize);
 	sb.sb_inprogress = 0;
-	libxfs_xlate_sb(block_buffer, &sb, 0, XFS_SB_ALL_BITS);
+	libxfs_sb_to_disk((xfs_dsb_t *)block_buffer, &sb);
+	if (xfs_sb_version_hascrc(&sb)) {
+		xfs_update_cksum(block_buffer, sb.sb_sectsize,
+				 offsetof(struct xfs_sb, sb_crc));
+	}
+
 	if (pwrite(dst_fd, block_buffer, sb.sb_sectsize, 0) < 0)
 		fatal("error writing primary superblock: %s\n", strerror(errno));
 
@@ -181,7 +193,7 @@ perform_restore(
 static void
 usage(void)
 {
-	fprintf(stderr, "Usage: %s [-bg] source target\n", progname);
+	fprintf(stderr, "Usage: %s [-V] [-g] source target\n", progname);
 	exit(1);
 }
 
